@import './_variables/general.scss';
@import './text.scss';

.react-layout-components--box {
    display: -webkit-box;
    display: -moz-box;
    display: -ms-flexbox;
    display: -webkit-flex;
    display: flex;
}

body, html {
    box-sizing: border-box;
    width: 100%;
    height: 100% !important;
    margin: 0;
    padding: 0;
}

#app {
    height: 100% !important;
}

a {
    color: #b63b3b;
    text-decoration: none;
    cursor: pointer;
}

.margin-top {
    margin-top: 20px !important;
}

.margin-left {
    margin-left: 15px !important;
}

.margin-right {
    margin-right: 15px !important;
}

.margin-bottom {
    margin-bottom: 20px !important;
}

.fullheight {
    height: 100% !important;
}

.table-row:hover {
    background-color: #f0f0f0;
}

.pointer {
    cursor: pointer;
}

.logo-container {
    position: relative;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    display: flex;
    align-items: center;
}

.logo {
    margin-left: 10px;
    height: 45px;
    vertical-align: middle;
}

.appbar {
    background-color: rgb(27, 28, 29) !important;
    color: #fafafa !important;
	-webkit-box-shadow: none !important;
	-moz-box-shadow: none !important;
    box-shadow: none !important;
    border-bottom: 1px solid rgb(27, 28, 29);
}

.vis-network:focus {
    outline: 0;
}
